#!/usr/bin/perl -w
# 
# sb::Plugin::CommentItemMust - Plugin Module for Serene Bach
# - Written by Gin-Lime / published on 2006-05-29
# -- `Serene Bach' is written by T.Otani
# --   Copyright (C) 2005- SimpleBoxes/SerendipityNZ.

package sb::Plugin::CommentItemMust;
# ==================================================
# // initialization for plugin
# ==================================================
use sb::Plugin ();
sb::Plugin->register_plugin(
	'lang' => {
		'ja' => 'euc',
		'en' => 'ascii',
	},
	'text' => {
		'type'    => 'admin, extra',
		'name'    => 'Comment Item Must',
		'text'    => 'Specifies author and e-mail indispensable when a comment posted.',
		'author'  => 'Gin-Lime',
		'detail'  => 'http://www.emeraldshell.com/sb/',
		'version' => '0.03',
		'publish' => 'yes',	# ץ饰ꥹȤΤγĥʸ/
	},
	'file' => 'commentitemmust.txt',
	'data' => 1,
);
# adminѥץ饰ȤϿ
sb::Plugin->register_admin_module(
	'mode'   => 'commentitemmust',
	'level'  => 2,
	'module' => 'sb::Admin::CommentItemMust',
);
# ץ饰ͭ˥С饤ɤ
sub _init {
	my $name = __FILE__;
	$name =~ s/.*[:\/\\](.*)/$1/;
	if(!&sb::Plugin::_ignore_plugin($name)) {
		use sb::Receipt();
		*sb::Receipt::_message = \&_message;
	}
}
&_init();
# ==================================================
# // function override
# ==================================================
use sb::Language ();
use sb::Receipt ();
use sb::Text ();
sub _message {# receiving comment, this function should return an entry object and error message
	my $self  = shift;

	my $set_data = sb::Plugin->get_data;
	my @settings = split('<>',$set_data->setting);
	my $auth_must = ($settings[0] ne '') ? $settings[0] : &sb::Admin::CommentItemMust::AUTH_MUST;
	my $email_must = ($settings[1] ne '') ? $settings[1] : &sb::Admin::CommentItemMust::EMAIL_MUST;
	my $use_trip = ($settings[2] ne '') ? $settings[2] : &sb::Admin::CommentItemMust::USE_TRIP;
	$use_trip = int($use_trip);
	my @strings = split('<>',$set_data->text);
	my $author_noname = ($strings[0] ne '') ? $strings[0] : &sb::Admin::CommentItemMust::AUTHOR_NONAME;
	$author_noname = sb::Text->detitize($author_noname);
	my $author_title = ($strings[1] ne '') ? $strings[1] : &sb::Admin::CommentItemMust::AUTHOR_TITLE;
	$author_title = sb::Text->detitize($author_title);
	my $email_title = ($strings[2] ne '') ? $strings[2] : &sb::Admin::CommentItemMust::EMAIL_TITLE;
	$email_title = sb::Text->detitize($email_title);

	my $entry = sb::Data->load('Entry','id'=>$self->{'id'});
	my $now   = $self->{'time'};
	my $cgi   = $self->{'cgi'};
	my $lang  = sb::Language->get;
	return($entry,"error_no_entry\n") if (!$entry);
	my $code = ($cgi->value('charset') eq '') # charcode not charset to use sb::Language
	         ? $lang->charcode 
	         : $lang->code_for_charset($cgi->value('charset'));
	$lang->checkcode($cgi->value('description'),$code);
	my %com = ( # buffering data for received comment
		'eid'  => $entry->id,
		'stat' => ($entry->acm == 1) ? 1 : 0,
		'date' => $now,
		'tz'   => sb::Config->get->value('conf_timezone'),
		'body' => &sb::Receipt::_convert_message_text('description',$cgi,$lang),
		'auth' => &sb::Receipt::_convert_message_text('name',$cgi,$lang),
		'mail' => $cgi->value('email'),
		'url'  => $cgi->value('url'),
		'host' => $cgi->value('_addr'),
		'agnt' => $cgi->value('_agnt'),
		'icon' => $cgi->value('icon'),
	);
	$com{'icon'} = int($com{'icon'}) if ($com{'icon'} ne '');
	my $num  = $entry->com;
	my @objs = &sb::Receipt::_load_objs('Message',$entry->id);

	my $error_no_author = my $error_no_email = $lang->string('error_no_comment_item_former') . '<>' . $lang->string('error_no_comment_item_latter') . "\n";
	$error_no_author =~ s/\<\>/$author_title/;
	$error_no_email =~ s/\<\>/$email_title/;

	eval { # error check
		die("error_no_comment\n") if ( $com{'body'} eq '' );
		die("error_banned\n")     if ( $entry->acm == 0 );
		die("error_no_entry\n")   if ( $entry->stat == 0 );
		die("error_banned\n")     if ( !&sb::Receipt::_check_ip($cgi->value('_addr')) );
		die("error_doubled\n")    if ( &sb::Receipt::_check_redundancy_for_com(\%com,@objs) );
	};
	return($entry,$@) if ($@);

	if($com{'auth'} eq '') {
		if($auth_must eq '0') {
			$com{'auth'} = $author_noname;
		} else {
			die($error_no_author);
		}
	} else {
		$com{'auth'} = &sb::Plugin::CommentItemMust::_trip($com{'auth'},$use_trip - 1) if($use_trip > 0);
	}
	die($error_no_email) if (($com{'mail'} eq '') && ($email_must eq '1'));

	my $new = sb::Data->add('Message',%com);
	my $stat = $self->_check_spam($entry,$new,$cgi->value('_refe'));
	$new->stat($stat);
	$entry->com( ++$num ) if ($new->stat == 1);
	$self->_sendmail('com',$new,$entry);
	sb::Data->update($new);
	if ($cgi->value('set_cookie')) { # set cookies
		my $conf = sb::Config->get;
		my %data = ();
		$lang->checkcode('',$code) if ($code ne COOKIE_CHARCODE);
		foreach my $key ( @{$conf->value('basic_cookie')} ) {
			my $value = ($key ne 'checkid') ? $cgi->value($key) : $conf->value('conf_spamid');
			$value = $lang->convert($value,COOKIE_CHARCODE) if ($code ne COOKIE_CHARCODE);
			$data{$key} = $value;
		}
		$cgi->set_cookie(
			'time'   => $now,
			'name'   => $conf->value('basic_cooktag'),
			'expire' => COOKIE_EXPIRES,
			'data'   => \%data,
		);
	}
	return ($new->stat == 1) ? ($entry,'') : ($entry,"error_wait_msg\n");
}
# ==================================================
# // private functions - other utilities
# ==================================================
sub _trip {
	my $trip_name = shift;
	my $hidden_trip = shift;
	$trip_name =~ s///;
	if ($trip_name =~ /(#|)(.+)$/) {
		my $tripkey = $2;
		my $salt = substr(substr($tripkey, 1, 2) . "H.", 0, 2);
		$salt =~ s/[^\.-z]/\./go;
		$salt =~ tr/:;<=>?@[\\]^_`/ABCDEFGabcdef/;
		my $tripstr = substr(crypt($tripkey, $salt),-10);
		$tripstr = ($hidden_trip) ? '<!-- Trip:'. $tripstr . ' -->' : ' ' . $tripstr;
		$trip_name =~ s/(#|)(.+)$/$tripstr/;
	}
	return($trip_name);
}

package sb::Admin::CommentItemMust;
use vars qw( @ISA );
use sb::Interface ();
use sb::TemplateManager ();
use sb::Text ();
@ISA = qw( sb::App::Admin );
# ==================================================
# // declaration for constant value
# ==================================================
sub TEMPLATE      (){ 'commentitemmust.html' };
# ̵̾ξν0:AUTHOR_NONAME ̾/1:ٹ𤹤
sub AUTH_MUST     (){ '1' };
# ᡼륢ɥ쥹ɬܤˤ뤫0:ǤƤǤ/1:ɬܡ
sub EMAIL_MUST     (){ '0' };
# ȥåפν0:Ѥʤ/1:̾θɽ/2:HTML ȡ
sub USE_TRIP      (){ '0' };
# ̵̾ξ̾
sub AUTHOR_NONAME (){ 'No Name' };
# ̾פΥȥ
sub AUTHOR_TITLE  (){ 'name' };
# E ᡼פΥȥ
sub EMAIL_TITLE   (){ 'email' };
# ==================================================
# // public functions - callback
# ==================================================
sub callback {
	my $self = shift;
	return ( $self->{'regi'} ) 
		? $self->_save_setting(@_)
		: $self->_show_setting(@_);
}
# ==================================================
# // private functions - main routine
# ==================================================
sub _save_setting {
	my $self = shift;
	my $cgi  = sb::Interface->get;
	# ǡμ
	my $data = sb::Plugin->get_data;

	# μ
	$data->setting($cgi->value('auth_must') . '<>' . $cgi->value('email_must') . '<>' . $cgi->value('use_trip'));
	my $author_noname = ($cgi->value('author_noname') ne '') ? $cgi->value('author_noname') : $cgi->value('author_noname_hidden');
	$author_noname = sb::Text->entitize($author_noname);
	my $author_title = ($cgi->value('author_title') ne '') ? $cgi->value('author_title') : $cgi->value('author_title_hidden');
	$author_title = sb::Text->entitize($author_title);
	my $email_title = ($cgi->value('email_title') ne '') ? $cgi->value('email_title') : $cgi->value('email_title_hidden');
	$email_title = sb::Text->entitize($email_title);
	$data->text($author_noname . '<>' . $author_title . '<>' . $email_title);

	# ǡ¸
	sb::Plugin->set_data('data' => $data);

	# ¸λåɽ
	my $lang = sb::Language->get;
	my $msg = $lang->string('parts_confcomp');
	return $self->_show_setting('message'=>$msg);
}
sub _show_setting {
	my $self = shift;
	my %param = ('message' => '', @_);
	my $cms  = sb::TemplateManager->new(sb::Plugin->load_template('file' => TEMPLATE));
	my $set_data = sb::Plugin->get_data;
	my @settings = split('<>',$set_data->setting);
	my $auth_must = ($settings[0] ne '') ? $settings[0] : AUTH_MUST;
	my $email_must = ($settings[1] ne '') ? $settings[1] : EMAIL_MUST;
	my $use_trip = ($settings[2] ne '') ? $settings[2] : USE_TRIP;
	my @strings = split('<>',$set_data->text);
	my $author_noname = ($strings[0] ne '') ? $strings[0] : AUTHOR_NONAME;
	my $author_title = ($strings[1] ne '') ? $strings[1] : AUTHOR_TITLE;
	my $email_title = ($strings[2] ne '') ? $strings[2] : EMAIL_TITLE;

	# ȼͤ
	$cms->num(0);
	$cms->tag('sb_cam_auth_must_0' =>
	          (!$auth_must) ? ' selected' : '' );
	$cms->tag('sb_cam_auth_must_1' =>
	          ($auth_must) ? ' selected' : '' );
	$cms->tag('sb_cam_email_must_0' =>
	          (!$email_must) ? ' selected' : '' );
	$cms->tag('sb_cam_email_must_1' =>
	          ($email_must) ? ' selected' : '' );
	$cms->tag('sb_cam_use_trip_0' =>
	          ($use_trip eq '0') ? ' selected' : '' );
	$cms->tag('sb_cam_use_trip_1' =>
	          ($use_trip eq '1') ? ' selected' : '' );
	$cms->tag('sb_cam_use_trip_2' =>
	          ($use_trip eq '2') ? ' selected' : '' );
	$cms->tag('sb_cam_author_noname' => $author_noname);
	$cms->tag('sb_cam_author_title' => $author_title);
	$cms->tag('sb_cam_email_title' => $email_title);
	$cms->tag('sb_cam_author_noname_hidden' => $author_noname);
	$cms->tag('sb_cam_author_title_hidden' => $author_title);
	$cms->tag('sb_cam_email_title_hidden' => $email_title);

	# åɽ
	if ($param{'message'} ne '') {
		$cms->num(0);
		$cms->tag('sb_process_message' => $param{'message'});
		$cms->block('sb_process_message' => 1);
	}

	# ̤ɽ
	return sb::Interface->get->head('type'=>'text/html') . $self->set_main($cms->output);
}

1;
# require 뤿Τޤʤ
__END__


Original functions overridden

*sb::Receipt
Version:0.14

sub _message { # receiving comment, this function should return an entry object and error message
	my $self  = shift;
	my $entry = sb::Data->load('Entry','id'=>$self->{'id'});
	my $now   = $self->{'time'};
	my $cgi   = $self->{'cgi'};
	my $lang  = sb::Language->get;
	return($entry,"error_no_entry\n") if (!$entry);
	my $code = ($cgi->value('charset') eq '') # charcode not charset to use sb::Language
	         ? $lang->charcode 
	         : $lang->code_for_charset($cgi->value('charset'));
	$lang->checkcode($cgi->value('description'),$code);
	my %com = ( # buffering data for received comment
		'eid'  => $entry->id,
		'stat' => ($entry->acm == 1) ? 1 : 0,
		'date' => $now,
		'tz'   => sb::Config->get->value('conf_timezone'),
		'body' => &_convert_message_text('description',$cgi,$lang),
		'auth' => &_convert_message_text('name',$cgi,$lang),
		'mail' => $cgi->value('email'),
		'url'  => $cgi->value('url'),
		'host' => $cgi->value('_addr'),
		'agnt' => $cgi->value('_agnt'),
		'icon' => $cgi->value('icon'),
	);
	$com{'icon'} = int($com{'icon'}) if ($com{'icon'} ne '');
	my $num  = $entry->com;
	my @objs = &_load_objs('Message',$entry->id);
	eval { # error check
		die("error_no_comment\n") if ( $com{'body'} eq '' );
		die("error_banned\n")     if ( $entry->acm == 0 );
		die("error_no_entry\n")   if ( $entry->stat == 0 );
		die("error_banned\n")     if ( !&_check_ip($cgi->value('_addr')) );
		die("error_doubled\n")    if ( &_check_redundancy_for_com(\%com,@objs) );
	};
	return($entry,$@) if ($@);
	my $new = sb::Data->add('Message',%com);
	my $stat = $self->_check_spam($entry,$new,$cgi->value('_refe'));
	$new->stat($stat);
	$entry->com( ++$num ) if ($new->stat == 1);
	$self->_sendmail('com',$new,$entry);
	sb::Data->update($new);
	if ($cgi->value('set_cookie')) { # set cookies
		my $conf = sb::Config->get;
		my %data = ();
		$lang->checkcode('',$code) if ($code ne COOKIE_CHARCODE);
		foreach my $key ( @{$conf->value('basic_cookie')} ) {
			my $value = ($key ne 'checkid') ? $cgi->value($key) : $conf->value('conf_spamid');
			$value = $lang->convert($value,COOKIE_CHARCODE) if ($code ne COOKIE_CHARCODE);
			$data{$key} = $value;
		}
		$cgi->set_cookie(
			'time'   => $now,
			'name'   => $conf->value('basic_cooktag'),
			'expire' => COOKIE_EXPIRES,
			'data'   => \%data,
		);
	}
	return ($new->stat == 1) ? ($entry,'') : ($entry,"error_wait_msg\n");
}

